import argparse
from pathlib import Path
from .changelog_helpers.helpers import (
    get_latest_version,
)
from .changelog_helpers.summarize import get_summary


def add_changelog_path_argument(
    parser: argparse.ArgumentParser,
) -> argparse.ArgumentParser:
    """
    Add a positional argument for the changelog path.
    """

    return parser.add_argument(
        "changelog_path",
        nargs="?",
        default=Path.cwd(),
        help="Path to the changelog file to process. Default path is the current directory. "
        "If the filename is not provided, it'll look for a CHANGELOG.md file.",
    )


def add_include_unreleased_argument(
    parser: argparse.ArgumentParser,
) -> argparse.ArgumentParser:
    """
    Add an argument to be able to include Unreleased section when perform actions on a changelog file.
    """

    return parser.add_argument(
        "--include_unreleased",
        default=False,
        action="store_true",
        help="Include unreleased items to the output. By default, only released items are included.",
    )


def add_get_parser(sub_commands_parser: argparse.ArgumentParser):
    """
    Define command line arguments for get feature.
    """

    get_parser = sub_commands_parser.add_parser("get", help="Check and display the latest version of a changelog file.")
    # Add argument to include Unreleased section of changelog or not
    add_include_unreleased_argument(get_parser)
    # Add argument to be able to choose another path for changelog file
    add_changelog_path_argument(get_parser)

    return get_parser


def get(arguments: argparse.Namespace):
    """
    :return: latest version of a changelog file.
    """

    print(get_latest_version(arguments.changelog_path, arguments.include_unreleased))


def add_summarize_parser(sub_commands_parser: argparse.ArgumentParser):
    """
    Define command line arguments for summarize feature.
    """

    summarize_parser = sub_commands_parser.add_parser(
        "summarize",
        help="Check and display the changes list between two versions of a changelog file.",
    )

    summarize_parser.add_argument(
        "--old",
        help="Version from which we want the changes list. Default old version is the initial version of the changelog file.",
    )
    summarize_parser.add_argument(
        "--new",
        help="Version to which we want the changes list. Default new version is the latest version of the changelog file.",
    )
    # Add argument to include Unreleased section of changelog or not
    add_include_unreleased_argument(summarize_parser)
    # Add argument to be able to choose another path for changelog file
    add_changelog_path_argument(summarize_parser)

    return summarize_parser


def summarize(arguments: argparse.Namespace):
    """
    :return: the changes list between two versions, provided or not, of a changelog file.
    """

    summaries, unassigned_items, old, new = get_summary(
        arguments.changelog_path, arguments.old, arguments.new, arguments.include_unreleased
    )

    print(f"# Changelog summary\n\nHere are all the changes between {old} and {new}:")
    # Display dict with changes sections sorted by alphabetic order
    for summary in sorted(summaries.keys()):
        print(f"\n{summary}\n")
        for change in summaries[summary]:
            print(change)
    if unassigned_items:
        print("\n/! WARNING: Following changes don't belong to any section:\n")
        for item in unassigned_items:
            print(item)


def main():
    import sys
    import argparse

    # Set up top level parser
    parser = argparse.ArgumentParser()

    # Set up sub_commands parser
    sub_commands = parser.add_subparsers(dest="command", title="Available commands")
    # Add all commands and their parameters
    add_get_parser(sub_commands)
    add_summarize_parser(sub_commands)

    arguments = parser.parse_args()

    try:
        if "get" == arguments.command:
            get(arguments)

        elif "summarize" == arguments.command:
            summarize(arguments)

        else:
            parser.print_help(sys.stderr)
            sys.exit(1)

    except Exception as e:
        print("An error occurred:", e, file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



import re
from pathlib import Path
from typing import List


def sanitize_changelog_path(changelog_path: Path) -> Path:
    """
    Check that the given path points to a valid file and if a file named CHANGELOG.md exists in the given directory.

    :param Path changelog_path: Path of the changelog we want to extract parts.
    :return: A valid changelog path.
    """

    if not changelog_path.is_file():
        changelog_path = changelog_path / "CHANGELOG.md"

    if not changelog_path.is_file():
        raise FileNotFoundError(f"The changelog file {changelog_path} doesn't exist.")

    return changelog_path


def read_changelog_path(changelog_path: Path) -> List[str]:
    """
    Open and read a changelog path.

    :param Path changelog_path: Path of the changelog we want to extract parts.
    :return: A splited changelog file.
    """

    changelog_file = changelog_path.read_text()

    return changelog_file.split("\n")


def get_version_line_pattern(changelog_format: str = "markdown") -> str:
    """
    :return: Version line pattern depending on changelog file format.
    """

    if changelog_format == "markdown":
        return r"^## \[(.+)\]"


def get_version_pattern(version: str) -> str:
    """
    Use a provided version to create a pattern to help finding version number in a changelog file.

    :return: Version pattern.
    """

    return rf"^## \[{re.escape(version)}\]"


def _get_version_helper(changelog_path: Path | str, include_unreleased: bool = False, get_initial: bool = False) -> str:
    """
    Find and return current version of a changelog file.

    :param Path | str changelog_path: Path of the changelog we want to extract parts.
    :param bool include_unreleased: If set to True, Unreleased section will be taken into account.
    :param bool get_initial: If set to True, return value will be the initial version of the provided changelog file.
    :return: Latest version of the provided changelog file.
    :raises RuntimeError: If the file doesn't contain any version line.
    """

    path = sanitize_changelog_path(Path(changelog_path))

    changelog_file = read_changelog_path(path)

    if get_initial:
        changelog_file = reversed(changelog_file)

    for line in changelog_file:
        current_version = re.search(get_version_line_pattern(), line)

        # If current_version isn't found or we found Unreleased section we continue, unless include_unreleased is set to True
        if current_version is None or (current_version.group(1) == "Unreleased" and not include_unreleased):
            continue
        return current_version.group(1)
    # If no version has been found, we throw an error.
    raise RuntimeError("Couldn't find a version line in the file")


def get_latest_version(changelog_path: Path | str, include_unreleased: bool = False) -> str:
    """
    :param Path | str changelog_path: Path of the changelog we want to extract parts.
    :param bool include_unreleased: If set to True, Unreleased section will be taken into account.
    :return: Latest version of a changelog file.
    """

    return _get_version_helper(changelog_path, include_unreleased)


def get_initial_version(changelog_path: Path | str, include_unreleased: bool = False) -> str:
    """
    :param Path | str changelog_path: Path of the changelog we want to extract parts.
    :param bool include_unreleased: If set to True, Unreleased section will be taken into account.
    :return: Initial version of a changelog file.
    """

    return _get_version_helper(changelog_path, include_unreleased, True)


def is_version_in_changelog(changelog_path: Path | str, version: str) -> bool:
    """
    Check if given version exists in the given changelog file.

    :param Path | str changelog_path: Path of the changelog we want to extract parts.
    :return: True if the provided version number exists, otherwise return False.
    """

    changelog_file = read_changelog_path(Path(changelog_path))

    for line in changelog_file:
        if re.search(rf"^## \[{version}]", line) is None:
            continue
        return True

    return False


def is_version_line(line: str) -> bool:
    """
    Check if the line is a line that contains a version.

    :return: True if the line contains a version, otherwise return False.
    """

    if line.startswith("## ["):
        return True

    return False


def is_section_line(line: str) -> bool:
    """
    Check if the line is a section heading.

    :return: True if the line is a heading, otherwise return False.
    """

    if line.startswith("###"):
        return True

    return False
